// Copyright 2022 Open Source Robotics Foundation, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <string.h>

#include "exlib/exlib.h"

int main(int argc, char * argv[])
{
  int should_fail = 0;

  if (argc >= 2 && !strcmp("should-fail", argv[1])) {
    should_fail = 1;
  }

  if (get_fourty_two() != 42) {
    return should_fail ? 0 : 42;
  }

  if (get_twenty_one() != 21) {
    return 21;
  }

  return should_fail ? 42 : 0;
}
