// Copyright 2022 Open Source Robotics Foundation, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef EXLIB__EXLIB_H_
#define EXLIB__EXLIB_H_

#if __cplusplus
extern "C"
{
#endif

#include "exlib/visibility_control.h"

EXLIB_PUBLIC int get_fourty_two();

EXLIB_PUBLIC int get_twenty_one();

#if __cplusplus
}
#endif

#endif  // EXLIB__EXLIB_H_
